#ifndef POKEPLATINUM_OV35_02256410_H
#define POKEPLATINUM_OV35_02256410_H

#include "struct_decls/struct_02018340_decl.h"
#include "overlay035/struct_ov35_02256410_1.h"
#include "overlay035/struct_ov35_02256410_decl.h"

BOOL ov35_02256410(UnkStruct_ov35_02256410 ** param0, const UnkStruct_ov35_02256410_1 * param1, UnkStruct_02018340 * param2);
void ov35_0225644C(UnkStruct_ov35_02256410 * param0);
void ov35_02256548(UnkStruct_ov35_02256410 * param0, u32 param1);
BOOL ov35_0225656C(UnkStruct_ov35_02256410 * param0, u32 param1);
BOOL ov35_02256578(UnkStruct_ov35_02256410 * param0);

#endif // POKEPLATINUM_OV35_02256410_H
